/*
 * This file is part of the Sejda source code
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.sejda.cli;

import java.util.ArrayList;
import java.util.List;

import org.sejda.model.notification.EventListener;
import org.sejda.model.notification.event.TaskExecutionWarningEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DefaultTaskWarningsEventListener implements EventListener<TaskExecutionWarningEvent> {
    private static final Logger LOG = LoggerFactory.getLogger(DefaultTaskWarningsEventListener.class);

    private List<String> warnings = new ArrayList<>();

    @Override
    public void onEvent(TaskExecutionWarningEvent event) {
        LOG.warn(event.getWarning());

        warnings.add(event.getWarning());
    }

    public List<String> getWarnings() {
        return warnings;
    }
}
